;;*****************************************************************************
;;
;; File: BubblerLoadler.asm
;;
;; See comments in Bubbler.asm for conversion from PDS to DASM
;;
;; This now split out into separate program
;;
;;*****************************************************************************
        PROCESSOR 6502

;
; Constants
;
LOADER_LOCATION EQU $0400

        INCLUDE CbmBasicTokens.asm
        INCLUDE C64KernalEntryPoints.asm

        ;
        ; BASIC lines to run code below and then load & run main application
        ;
        ORG $0801
BASIC_L10
        WORD BASIC_L20 ; Address of next line?
        WORD 10 ; Line number
        BYTE BASIC_TKN_SYS ; SYS command
        BYTE " "
        BYTE [CODE_START/1000 + $30] ; Address (in ASCII)
        BYTE [[CODE_START%1000]/100 + $30]
        BYTE [[CODE_START%100]/10 + $30]
        BYTE [[CODE_START%10] + $30]
        BYTE $00 ; End of line
BASIC_L20
        WORD BASIC_END ; Address of next line?
        WORD 20 ; Line number
        BYTE BASIC_TKN_LOAD
        BYTE " "
        BYTE '"
        BYTE "BUBBLER-MAIN*" ; Inside code, ASCII characters used for letters
        BYTE '"
        BYTE ",8,1"
        BYTE $00 ; End of line
BASIC_END
        WORD $0000

;
;
; Following is from http://sta.c64.org/cbm64mem.html
;
; Location $0001 is one of the processor ports on the C64:
;
;  - Bits #0-#2: Configuration for memory areas $A000-$BFFF
;                                               $D000-$DFFF
;                                               $E000-$FFFF
;    Values: - %x00: RAM visible in all three areas.
;            - %x01: RAM visible at $A000-$BFFF and $E000-$FFFF.
;            - %x10: RAM visible at $A000-$BFFF; KERNAL ROM visible at $E000-$FFFF.
;            - %x11: BASIC ROM visible at $A000-$BFFF; KERNAL ROM visible at $E000-$FFFF.
;            - %0xx: Character ROM visible at $D000-$DFFF. (Except for the value %000, see above.)
;            - %1xx: I/O area visible at $D000-$DFFF. (Except for the value %100, see above.)
;
;  - Bit #3: Datasette output signal level.
;
;  - Bit #4: Datasette button status:
;              0 = One or more of PLAY, RECORD, F.FWD or REW pressed
;              1 = No button is pressed.
;
;  - Bit #5: Datasette motor control; 0 = On; 1 = Off.
;
; Default: $37, %00110111.
;
FROM_ADDR      EQU $FB
TO_ADDR        EQU FROM_ADDR+2

;LOAD_COL_LOCN  EQU $8C00
;LOAD_SCRN_LOCN EQU $A000
LOAD_COL_LOCN  EQU $DC00
LOAD_SCRN_LOCN EQU $E000

CODE_START
               JSR $E544    ; Clear screen

               SEI
               LDA #$35 ; %00110101 -- page ROMS out but leave IO area paged in
               STA $01

               LDA #0
               STA $D011    ; Screen off
               STA $D020    ; Screen border colour
               STA $D021    ; Screen background colour

               ;
               ; Following note is from http://www.pagetable.com/?p=53:
               ;
               ;   If the VIC is set to banks $0000 or $8000, then the two
               ;   built-in character sets shadow RAM in the area of
               ;   $1000-$1FFF. This means that the built-in character set can
               ;   be used on those banks without occupying RAM, but it also
               ;   means that the area from $1000-$1FFF cannot be used for
               ;   bitmap, screen RAM or sprite data either.
               ;
               ; VIC Bank #2, $8000-$BFFF, 32768-49151.
               ; As per note above, means we can't use $9000-$9FFF for bitmap
               ; or colour data
               LDA #$01
               STA $DD00

               ; Set bitmap screen at $A000-$BFFF, screen memory (colour for
               ; bitmap) at $8C00
               LDA #$38
               STA $D018

               ; VIC Bank #3, $C000-$FFFF, 49152-65535
               LDA #0
               STA $DD00
               ; Set bitmap screen at $E000-$FFFF, screen memory (colour for
               ; bitmap) at $DC00
               LDA #$78
               STA $D018

               ; Page ROMS and IO area out
               LDA #$34
               STA $01

               LDA #<LOADING_SCREEN
               STA FROM_ADDR
               LDA #>LOADING_SCREEN
               STA FROM_ADDR+1
               LDA #<LOAD_COL_LOCN
               STA TO_ADDR
               LDA #>LOAD_COL_LOCN
               STA TO_ADDR+1

               ; Calculate how many whole pages to copy
               LDX #>$0400
               LDA #<$0400
               JSR COPY_MEMORY

               LDA #<[LOADING_SCREEN + $0400]
               STA FROM_ADDR
               LDA #>[LOADING_SCREEN + $0400]
               STA FROM_ADDR+1
               LDA #<LOAD_SCRN_LOCN
               STA TO_ADDR
               LDA #>LOAD_SCRN_LOCN
               STA TO_ADDR+1

               ; Calculate how many whole pages to copy
               LDX #>[LOADING_SCREEN_END - LOADING_SCREEN - $0400]
               LDA #<[LOADING_SCREEN_END - LOADING_SCREEN - $0400]
               JSR COPY_MEMORY

               LDA #$37 ; page IO area and ROMS back in
               STA $01

               LDA #$3B
               STA $D011    ; Turn screen on in bitmap mode

               ;
               ; Copy the loader code to where it won't get overwritten by the
               ; main program as it's loading in.
               ;
               ; We could copy a little less code (eg file open etc could be
               ; done here), but it's a little neater to keep all the loading
               ; stuff together.
               ;
               LDA #<LOADER_CODE
               STA FROM_ADDR
               LDA #>LOADER_CODE
               STA FROM_ADDR+1
               LDA #<LOADER_LOCATION
               STA TO_ADDR
               LDA #>LOADER_LOCATION
               STA TO_ADDR+1
               LDX #>[LOADER_CODE_END - LOADER_CODE]
               LDA #<[LOADER_CODE_END - LOADER_CODE]
               JSR COPY_MEMORY
               CLI

               JMP LOADER_LOCATION


;*******************************************************************************
;* Copy a block of memory.  Assumes that blocks do not overlap.
;*
;* Inputs: FROM_ADDR
;*         TO_ADDR
;*         X-register - number of whole pages to copy
;*         Accumulator - size of last partial page to copy
;*
;*******************************************************************************
COPY_MEMORY    CPX #0
               BEQ CM_ONLY_LAST_PG
               PHA
               LDY #0
CM_INNER_LOOP  LDA (FROM_ADDR),Y
               STA (TO_ADDR),Y
               INY
               BNE CM_INNER_LOOP ; Copy whole pages until we reach the last one
               INC FROM_ADDR+1
               INC TO_ADDR+1
               DEX
               BNE CM_INNER_LOOP
               PLA
               BEQ CM_DONE
CM_ONLY_LAST_PG
               TAX
CM_LAST_LOOP   LDA (FROM_ADDR),Y
               STA (TO_ADDR),Y
               INY
               DEX
               BNE CM_LAST_LOOP
CM_DONE        RTS


LOADER_CODE
;;******** vvv Relocated code vvv *********************************************
               ;
               ; This code is going to be copied to LOADER_LOCATION and run there
               ;
               RORG LOADER_LOCATION
               ; Turn off control and error messages
               LDA #$00
               JSR C64_KERN_SETMSG

               ; Set logical file number, device address, secondary address (command number)
               LDA #$08 ; logical file number
               TAY      ; secondary address (command number)
               TAX      ; device number
               JSR C64_KERN_SETLFS

               ; Set the name of the file we want to load in
               LDA #[MAIN_PROG_NAME_END - MAIN_PROG_NAME]
               LDX #<MAIN_PROG_NAME
               LDY #>MAIN_PROG_NAME
               JSR C64_KERN_SETNAM

               ; Open the file
               JSR C64_KERN_OPEN

               ; Command serial bus device #8 to talk
               LDA #$08
               JSR C64_KERN_TALK

               ; Set the secondary address to the device commanded to TALK
               LDA #$68
               JSR C64_KERN_TKSA

               ; Clear serial bus device status flags and serial bus output cache
               ; status.
               LDA #$00
               STA $90
               STA $94

               ; Read the first two bytes of the file, which are its start address in
               ; memory.
               JSR C64_KERN_ACPTR ; read a byte from the serial bus
               STA TO_ADDR
               JSR C64_KERN_ACPTR ; read a byte from the serial bus
               STA TO_ADDR+1

               ; Loop round and read the data from the file.  Note that calling ACPTR
               ; does not affect the Y register.
               LDY #0
LOOP1          JSR C64_KERN_ACPTR
               SEI
               STA (TO_ADDR),Y
               STY $D020
               LDA #0
               STA $D020
               INY
               BNE NOHI1
               INC TO_ADDR+1
NOHI1
               ; Check for end of file
               LDA $90
               AND #$40
               BEQ LOOP1

               ; Close logical file 8
               LDA #$08
               JSR C64_KERN_CLOSE

               JSR C64_KERN_CLALL ; Close all files

               ; Rather than drop back out to BASIC and run the
               JMP $080D


MAIN_PROG_NAME  BYTE "BUBBLER-MAIN.PRG"
MAIN_PROG_NAME_END

               REND
;;******** ^^^ Relocated code ^^^ *********************************************
LOADER_CODE_END


LOADING_SCREEN
        INCBIN bootpic.bin
LOADING_SCREEN_END

               IF LOADING_SCREEN_END > LOAD_SCRN_LOCN
               ERROR OVERLAP OF LOAD SCREEN FROM AND TO LOCATIONS!
               ENDIF

